@@ -18,41 +18,48 @@
 #' \describe{
 #'   \item{expressions}{a `list` of
 #'   `n+1` objects. The first `n` elements correspond to each expression in
-#'   `filename`, and consist of a list of 9 elements:
+#'   `filename`, and consist of a list of 8 elements:
 #'   \itemize{
-#'     \item{`filename` (`character`)}
-#'     \item{`line` (`integer`) the line in `filename` where this expression begins}
-#'     \item{`column` (`integer`) the column in `filename` where this expression begins}
+#'     \item{`filename` (`character`) the name of the file.}
+#'     \item{`line` (`integer`) the line in the file where this expression begins.}
+#'     \item{`column` (`integer`) the column in the file where this expression begins.}
 #'     \item{`lines` (named `character`) vector of all lines spanned by this
-#'           expression, named with the line number corresponding to `filename`}
-#'     \item{`parsed_content` (`data.frame`) as given by [utils::getParseData()] for this expression}
-#'     \item{`xml_parsed_content` (`xml_document`) the XML parse tree of this
-#'          expression as given by [xmlparsedata::xml_parse_data()]}
-#'     \item{`content` (`character`) the same as `lines` as a single string (not split across lines)}
+#'           expression, named with the corresponding line numbers.}
+#'     \item{`parsed_content` (`data.frame`) as given by [utils::getParseData()] for this expression.}
+#'     \item{`xml_parsed_content` (`xml_document`) the XML parse tree of this expression as given by
+#'           [xmlparsedata::xml_parse_data()].}
+#'     \item{`content` (`character`) the same as `lines` as a single string (not split across lines).}
+#'     \item{`xml_find_function_calls(function_names)` (`function`) a function that returns all `SYMBOL_FUNCTION_CALL`
+#'           XML nodes from `xml_parsed_content` with specified function names.}
 #'   }
 #'
 #'   The final element of `expressions` is a list corresponding to the full file
-#'   consisting of 6 elements:
+#'   consisting of 7 elements:
 #'   \itemize{
-#'     \item{`filename` (`character`)}
-#'     \item{`file_lines` (`character`) the [readLines()] output for this file}
+#'     \item{`filename` (`character`) the name of this file.}
+#'     \item{`file_lines` (`character`) the [readLines()] output for this file.}
 #'     \item{`content` (`character`) for .R files, the same as `file_lines`;
-#'           for .Rmd or .qmd scripts, this is the extracted R source code (as text)}
+#'           for .Rmd or .qmd scripts, this is the extracted R source code (as text).}
 #'     \item{`full_parsed_content` (`data.frame`) as given by
-#'           [utils::getParseData()] for the full content}
+#'           [utils::getParseData()] for the full content.}
 #'     \item{`full_xml_parsed_content` (`xml_document`) the XML parse tree of all
-#'           expressions as given by [xmlparsedata::xml_parse_data()]}
+#'           expressions as given by [xmlparsedata::xml_parse_data()].}
 #'     \item{`terminal_newline` (`logical`) records whether `filename` has a terminal
-#'           newline (as determined by [readLines()] producing a corresponding warning)}
+#'           newline (as determined by [readLines()] producing a corresponding warning).}
+#'     \item{`xml_find_function_calls(function_names)` (`function`) a function that returns all `SYMBOL_FUNCTION_CALL`
+#'           XML nodes from `full_xml_parsed_content` with specified function names.}
 #'   }
 #'   }
 #'   \item{error}{A `Lint` object describing any parsing error.}
+#'   \item{warning}{A `lints` object describing any parsing warning.}
 #'   \item{lines}{The [readLines()] output for this file.}
 #' }
 #'
-#' @examplesIf requireNamespace("withr", quietly = TRUE)
-#' tmp <- withr::local_tempfile(lines = c("x <- 1", "y <- x + 1"))
+#' @examples
+#' tmp <- tempfile()
+#' writeLines(c("x <- 1", "y <- x + 1"), tmp)
 #' get_source_expressions(tmp)
+#' unlink(tmp)
 #' @export
 get_source_expressions <- function(filename, lines = NULL) {
   source_expression <- srcfile(filename, encoding = settings$encoding)
@@ -61,17 +68,12 @@ get_source_expressions <- function(filename, lines = NULL) {
   old_lang <- set_lang("en")
   on.exit(reset_lang(old_lang))
 
-  source_expression$lines <- if (is.null(lines)) {
-    read_lines(filename)
-  } else {
-    lines
-  }
+  source_expression$lines <- lines %||% read_lines(filename)
 
   # Only regard explicit attribute terminal_newline=FALSE as FALSE and all other cases (e.g. NULL or TRUE) as TRUE.
-  # We don't use isFALSE since it is introduced in R 3.5.0.
-  terminal_newline <- !identical(attr(source_expression$lines, "terminal_newline", exact = TRUE), FALSE)
+  terminal_newline <- !isFALSE(attr(source_expression$lines, "terminal_newline", exact = TRUE))
 
-  e <- NULL
+  e <- w <- NULL
   source_expression$lines <- extract_r_source(
     filename = source_expression$filename,
     lines = source_expression$lines,
@@ -81,44 +83,44 @@ get_source_expressions <- function(filename, lines = NULL) {
   source_expression$content <- get_content(source_expression$lines)
   parsed_content <- get_source_expression(source_expression, error = function(e) lint_parse_error(e, source_expression))
 
-  if (inherits(e, "lint") && (is.na(e$line) || !nzchar(e$line) || e$message == "unexpected end of input")) {
+  # Currently no way to distinguish the source of the warning
+  #   from the message itself, so we just grep the source for the
+  #   exact string generating the warning; de-dupe in case of
+  #   multiple exact matches like '1e-3L; 1e-3L'.
+  # See https://bugs.r-project.org/show_bug.cgi?id=18863.
+  w <- lint_parse_warnings(w, parsed_content, source_expression)
+
+  if (is_lint(e) && (is.na(e$line) || !nzchar(e$line) || e$message == "unexpected end of input")) {
     # Don't create expression list if it's unreliable (invalid encoding or unhandled parse error)
-    expressions <- list()
-  } else {
-    top_level_map <- generate_top_level_map(parsed_content)
-    xml_parsed_content <- safe_parse_to_xml(parsed_content)
-
-    expressions <- lapply(
-      X = top_level_expressions(parsed_content),
-      FUN = get_single_source_expression,
-      parsed_content,
-      source_expression,
-      filename,
-      top_level_map
-    )
-
-    if (!is.null(xml_parsed_content) && !is.na(xml_parsed_content)) {
-      expression_xmls <- lapply(
-        xml_find_all(xml_parsed_content, "/exprlist/*"),
-        function(top_level_expr) xml2::xml_add_parent(xml2::xml_new_root(top_level_expr), "exprlist")
-      )
-      for (i in seq_along(expressions)) {
-        expressions[[i]]$xml_parsed_content <- expression_xmls[[i]]
-      }
-    }
-
-    # add global expression
-    expressions[[length(expressions) + 1L]] <- list(
-      filename = filename,
-      file_lines = source_expression$lines,
-      content = source_expression$lines,
-      full_parsed_content = parsed_content,
-      full_xml_parsed_content = xml_parsed_content,
-      terminal_newline = terminal_newline
-    )
+    return(list(expressions = list(), error = e, warning = w, lines = source_expression$lines))
   }
 
-  list(expressions = expressions, error = e, lines = source_expression$lines)
+  top_level_map <- generate_top_level_map(parsed_content)
+  xml_parsed_content <- safe_parse_to_xml(parsed_content)
+
+  expressions <- lapply(
+    X = top_level_expressions(parsed_content),
+    FUN = get_single_source_expression,
+    parsed_content,
+    source_expression,
+    filename,
+    top_level_map
+  )
+
+  expressions <- maybe_append_expression_xml(expressions, xml_parsed_content)
+
+  # add global expression
+  expressions[[length(expressions) + 1L]] <- list(
+    filename = filename,
+    file_lines = source_expression$lines,
+    content = source_expression$lines,
+    full_parsed_content = parsed_content,
+    full_xml_parsed_content = xml_parsed_content,
+    xml_find_function_calls = build_xml_find_function_calls(xml_parsed_content),
+    terminal_newline = terminal_newline
+  )
+
+  list(expressions = expressions, error = e, warning = w, lines = source_expression$lines)
 }
 
 lint_parse_error <- function(e, source_expression) {
@@ -150,6 +152,56 @@ lint_parse_error <- function(e, source_expression) {
   lint_parse_error_nonstandard(e, source_expression)
 }
 
+#' Currently no way to distinguish the source of the warning
+#'   from the message itself, so we just grep the source for the
+#'   exact string generating the warning; de-dupe in case of
+#'   multiple exact matches like '1e-3L; 1e-3L'
+#' @noRd
+lint_parse_warnings <- function(w, parsed_content, source_expression) {
+  if (!length(w)) {
+    return(w)
+  }
+  flatten_lints(lapply(unique(w), lint_parse_warning, parsed_content, source_expression))
+}
+
+#' The set of parser warnings seems pretty stable, but as
+#'   long as they don't generate sourceref hints, we're
+#'   stuck with this somewhat-manual approach.
+#' @noRd
+lint_parse_warning <- function(w, parsed_content, source_expression) {
+  for (lint_re in parser_warning_regexes) {
+    bad_txt <- re_matches(w, lint_re)$txt[1L]
+    # at most one regex matches the warning
+    if (is.na(bad_txt)) {
+      next
+    } else {
+      break
+    }
+  }
+  # use the parse tree to avoid baroque matches to comments, strings
+  hits <- parsed_content[with(parsed_content, token == "NUM_CONST" & text == bad_txt), ]
+  lapply(seq_len(nrow(hits)), function(ii) {
+    Lint(
+      filename = source_expression$filename,
+      line_number = hits$line1[ii],
+      column_number = hits$col1[ii],
+      type = "warning",
+      message = w,
+      line = source_expression$lines[[as.character(hits$line1[ii])]],
+      ranges = list(c(hits$col1[ii], hits$col2[ii]))
+    )
+  })
+}
+
+parser_warning_regexes <- list(
+  int_with_decimal =
+    rex("integer literal ", capture(anything, name = "txt"), " contains decimal; using numeric value"),
+  nonint_with_l =
+    rex("non-integer value ", capture(anything, name = "txt"), " qualified with L; using numeric value"),
+  unneeded_decimal =
+    rex("integer literal ", capture(anything, name = "txt"), " contains unnecessary decimal point")
+)
+
 #' Ensure a string is valid for printing
 #'
 #' Helper to ensure a valid string is provided as line where necessary.
@@ -197,10 +249,12 @@ lint_parse_error_r43 <- function(e, source_expression) {
     line_number <- line_number - 1L
   }
 
+  # Safely handle invalid location info
   if (line_number < 1L || line_number > length(source_expression$lines)) {
-    # Safely handle invalid location info
+    # nocov start
     line_number <- 1L
     column <- 1L
+    # nocov end
   }
 
   line <- fixup_line(source_expression$lines[[line_number]])
@@ -220,7 +274,7 @@ lint_parse_error_r43 <- function(e, source_expression) {
 
 #' Convert a R < 4.3.0 standard parse error message into a lint
 #'
-#' @param message_info Match of the structured parse error message regex, matched in [lint_parse_error()]
+#' @param message_info Match of the structured parse error message regex, matched in `lint_parse_error()`
 #' @param source_expression The source expression that generated the parse error
 #'
 #' @return A [Lint()] based on text mining of the error message captured by `message_info`,
@@ -477,6 +531,8 @@ get_single_source_expression <- function(loc,
     lines = expr_lines,
     parsed_content = pc,
     xml_parsed_content = xml2::xml_missing(),
+    # Placeholder for xml_find_function_calls, if needed (e.g. on R <= 4.0.5 with input source "\\")
+    xml_find_function_calls = build_xml_find_function_calls(xml2::xml_missing()),
     content = content
   )
 }
@@ -484,29 +540,24 @@ get_single_source_expression <- function(loc,
 get_source_expression <- function(source_expression, error = identity) {
   parse_error <- FALSE
 
-  parsed_content <- tryCatch(
-    parse(
-      text = source_expression$content,
-      srcfile = source_expression,
-      keep.source = TRUE
+  env <- parent.frame() # nolint: object_usage_linter. Used below.
+  # https://adv-r.hadley.nz/conditions.html
+  parsed_content <- withCallingHandlers(
+    tryCatch(
+      parse(
+        text = source_expression$content,
+        srcfile = source_expression,
+        keep.source = TRUE
+      ),
+      error = error
     ),
-    error = error
+    warning = function(w) {
+      env$w <- c(env$w, conditionMessage(w))
+      invokeRestart("muffleWarning")
+    }
   )
 
-  # TODO: Remove when minimum R version is bumped to > 3.5
-  #
-  # This needs to be done twice to avoid a bug fixed in R 3.4.4
-  #   https://bugs.r-project.org/bugzilla/show_bug.cgi?id=16041
-  parsed_content <- tryCatch(
-    parse(
-      text = source_expression$content,
-      srcfile = source_expression,
-      keep.source = TRUE
-    ),
-    error = error
-  )
-
-  if (inherits(parsed_content, c("error", "lint"))) {
+  if (is_error(parsed_content) || is_lint(parsed_content)) {
     assign("e", parsed_content, envir = parent.frame())
     parse_error <- TRUE
   }
@@ -517,7 +568,7 @@ get_source_expression <- function(source_expression, error = identity) {
     error = error
   )
 
-  if (inherits(parsed_content, c("error", "lint"))) {
+  if (is_error(parsed_content) || is_lint(parsed_content)) {
     # Let parse errors take precedence over encoding problems
     if (!parse_error) assign("e", parsed_content, envir = parent.frame())
     return() # parsed_content is unreliable if encoding is invalid
@@ -527,6 +578,21 @@ get_source_expression <- function(source_expression, error = identity) {
   fix_octal_escapes(fix_eq_assigns(fix_tab_indentations(source_expression)), source_expression$lines)
 }
 
+maybe_append_expression_xml <- function(expressions, xml_parsed_content) {
+  if (is.null(xml_parsed_content) || is.na(xml_parsed_content)) {
+    return(expressions)
+  }
+  expression_xmls <- lapply(
+    xml_find_all(xml_parsed_content, "/exprlist/*"),
+    function(top_level_expr) xml2::xml_add_parent(xml2::xml_new_root(top_level_expr), "exprlist")
+  )
+  for (i in seq_along(expressions)) {
+    expressions[[i]]$xml_parsed_content <- expression_xmls[[i]]
+    expressions[[i]]$xml_find_function_calls <- build_xml_find_function_calls(expression_xmls[[i]])
+  }
+  expressions
+}
+
 get_newline_locs <- function(x) {
   newline_search <- re_matches(x, rex("\n"), locations = TRUE, global = TRUE)[[1L]]$start
   c(
@@ -594,12 +660,14 @@ fix_tab_columns <- function(parse_content, tab_cols) {
 }
 
 tab_offsets <- function(tab_columns) {
-  cum_offset <- 0L
+  outer_env <- new.env(parent = emptyenv())
+  outer_env$cum_offset <- 0L
   vapply(
     tab_columns - 1L,
     function(tab_idx) {
-      offset <- 7L - (tab_idx + cum_offset) %% 8L # using a tab width of 8 characters
-      cum_offset <<- cum_offset + offset
+      # nolint next: object_overwrite_linter. 'offset' is a perfect name here.
+      offset <- 7L - (tab_idx + outer_env$cum_offset) %% 8L # using a tab width of 8 characters
+      outer_env$cum_offset <- outer_env$cum_offset + offset
       offset
     },
     integer(1L),
@@ -639,27 +707,15 @@ fix_eq_assigns <- function(pc) {
     parent = integer(n_expr),
     token = character(n_expr),
     terminal = logical(n_expr),
-    text = character(n_expr),
-    stringsAsFactors = FALSE
+    text = character(n_expr)
   )
 
   for (i in seq_len(n_expr)) {
-    start <- true_locs[i]
+    start_loc <- true_locs[i]
+    end_loc <- true_locs[i]
 
-    # TODO(michaelchirico): vectorize this loop away. the tricky part is,
-    #   this loop doesn't execute on most R versions (we tried 3.6.3 and 4.2.0).
-    #   so it likely requires some GHA print debugging -- tedious :)
-    end <- true_locs[i]
-    j <- end + 1L
-    # nocov start: only runs on certain R versions
-    while (j <= length(expr_locs) && !expr_locs[j]) {
-      end <- j
-      j <- j + 1L
-    }
-    # nocov end
-
-    prev_loc <- prev_locs[start]
-    next_loc <- next_locs[end]
+    prev_loc <- prev_locs[start_loc]
+    next_loc <- next_locs[end_loc]
 
     id_itr <- id_itr + 1L
     supplemental_content[i, ] <- list(
@@ -675,9 +731,9 @@ fix_eq_assigns <- function(pc) {
     )
 
     new_parent_locs <- c(
-      prev_locs[start:end],
-      eq_assign_locs[start:end],
-      next_locs[start:end],
+      prev_locs[start_loc:end_loc],
+      eq_assign_locs[start_loc:end_loc],
+      next_locs[start_loc:end_loc],
       next_loc
     )
     pc[new_parent_locs, "parent"] <- id_itr
