@@ -44,37 +44,36 @@
 #' @seealso [linters] for a complete list of linters available in lintr.
 #' @export
 redundant_ifelse_linter <- function(allow10 = FALSE) {
-  tf_xpath <- glue::glue("
-  //SYMBOL_FUNCTION_CALL[ {xp_text_in_table(ifelse_funs)} ]
-    /parent::expr
-    /parent::expr[
-      expr[NUM_CONST[text() = 'TRUE']]
-      and expr[NUM_CONST[text() = 'FALSE']]
-    ]
-  ")
+  tf_xpath <- glue("
+  parent::expr[
+    expr[position() <= 4 and NUM_CONST[text() = 'TRUE']]
+    and expr[position() <= 4 and NUM_CONST[text() = 'FALSE']]
+    and (
+      count(expr) = 4
+      or expr[5]/NUM_CONST[text() = 'NA']
+    )
+  ]")
 
-  num_xpath <- glue::glue("
-  //SYMBOL_FUNCTION_CALL[ {xp_text_in_table(ifelse_funs)} ]
-    /parent::expr
-    /parent::expr[
-      expr[NUM_CONST[text() = '1' or text() = '1L']]
-      and expr[NUM_CONST[text() = '0' or text() = '0L']]
-    ]
-  ")
+  num_xpath <- glue("
+  parent::expr[
+    expr[position() <= 4 and NUM_CONST[text() = '1' or text() = '1L']]
+    and expr[position() <= 4 and NUM_CONST[text() = '0' or text() = '0L']]
+    and (
+      count(expr) = 4
+      or expr[5]/NUM_CONST[text() = 'NA' or text() = 'NA_integer_' or text() = 'NA_real_']
+    )
+  ]")
 
-  Linter(function(source_expression) {
-    if (!is_lint_level(source_expression, "expression")) {
-      return(list())
-    }
+  Linter(linter_level = "expression", function(source_expression) {
+    xml_targets <- source_expression$xml_find_function_calls(ifelse_funs)
 
-    xml <- source_expression$xml_parsed_content
     lints <- list()
 
-    tf_expr <- xml2::xml_find_all(xml, tf_xpath)
+    tf_expr <- xml_find_all(xml_targets, tf_xpath)
     matched_call <- xp_call_name(tf_expr)
     # [1] call; [2] logical condition
-    first_arg <- xml2::xml_find_chr(tf_expr, "string(expr[3]/NUM_CONST)")
-    second_arg <- xml2::xml_find_chr(tf_expr, "string(expr[4]/NUM_CONST)")
+    first_arg <- xml_find_chr(tf_expr, "string(expr[3]/NUM_CONST)")
+    second_arg <- xml_find_chr(tf_expr, "string(expr[4]/NUM_CONST)")
     tf_message <- sprintf(
       "Just use the logical condition (or its negation) directly instead of calling %s(x, %s, %s)",
       matched_call, first_arg, second_arg
@@ -82,11 +81,11 @@ redundant_ifelse_linter <- function(allow10 = FALSE) {
     lints <- c(lints, xml_nodes_to_lints(tf_expr, source_expression, tf_message, type = "warning"))
 
     if (!allow10) {
-      num_expr <- xml2::xml_find_all(xml, num_xpath)
+      num_expr <- xml_find_all(xml_targets, num_xpath)
       matched_call <- xp_call_name(num_expr)
       # [1] call; [2] logical condition
-      first_arg <- xml2::xml_find_chr(num_expr, "string(expr[3]/NUM_CONST)")
-      second_arg <- xml2::xml_find_chr(num_expr, "string(expr[4]/NUM_CONST)")
+      first_arg <- xml_find_chr(num_expr, "string(expr[3]/NUM_CONST)")
+      second_arg <- xml_find_chr(num_expr, "string(expr[4]/NUM_CONST)")
       is_numeric_01 <- first_arg %in% c("0", "1") | second_arg %in% c("0", "1")
       coercion_function <- ifelse(is_numeric_01, "as.numeric", "as.integer")
       is_negated <- first_arg %in% c("0", "0L")
@@ -100,6 +99,6 @@ redundant_ifelse_linter <- function(allow10 = FALSE) {
       lints <- c(lints, xml_nodes_to_lints(num_expr, source_expression, lint_message, type = "warning"))
     }
 
-    return(lints)
+    lints
   })
 }
