@@ -31,8 +31,11 @@
 #' names(my_undesirable_functions)
 #' @export
 modify_defaults <- function(defaults, ...) {
-  if (missing(defaults) || !is.list(defaults) || !all(nzchar(names2(defaults)))) {
-    stop("`defaults` must be a named list.")
+  if (missing(defaults)) {
+    cli_abort("{.arg defaults} is a required argument, but is missing.")
+  }
+  if (!is.list(defaults) || !all(nzchar(names2(defaults)))) {
+    cli_abort("{.arg defaults} must be a named list, not {.obj_type_friendly {defaults}}.")
   }
   vals <- list(...)
   nms <- names2(vals)
@@ -43,12 +46,10 @@ modify_defaults <- function(defaults, ...) {
 
   to_null <- vapply(vals, is.null, logical(1L))
   if (!all(nms[to_null] %in% names(defaults))) {
-    bad_nms <- setdiff(nms[to_null], names(defaults))
-    is_are <- if (length(bad_nms) > 1L) "are" else "is"
-    warning(
-      "Trying to remove ", glue::glue_collapse(sQuote(bad_nms), sep = ", ", last = " and "),
-      ", which ", is_are, " not in `defaults`."
-    )
+    bad_nms <- setdiff(nms[to_null], names(defaults)) # nolint: object_usage_linter. TODO(#2252).
+    cli_warn(c(
+      i = "Trying to remove {.field {bad_nms}}, which {?is/are} not in {.arg defaults}."
+    ))
   }
 
   is.na(vals) <- nms == vals
@@ -91,8 +92,11 @@ modify_defaults <- function(defaults, ...) {
 #' }
 #' @export
 linters_with_tags <- function(tags, ..., packages = "lintr", exclude_tags = "deprecated") {
+  if (missing(tags)) {
+    cli_abort("{.arg tags} was not specified. Available tags: {available_tags()}")
+  }
   if (!is.character(tags) && !is.null(tags)) {
-    stop("`tags` must be a character vector, or NULL.")
+    cli_abort("{.arg tags} must be a character vector, or {.code NULL}, not {.obj_type_friendly {tags}}.")
   }
   tagged_linters <- list()
 
@@ -102,11 +106,11 @@ linters_with_tags <- function(tags, ..., packages = "lintr", exclude_tags = "dep
     available <- available_linters(packages = package, tags = tags, exclude_tags = exclude_tags)
     if (nrow(available) > 0L) {
       if (!all(available$linter %in% ns_exports)) {
-        missing_linters <- setdiff(available$linter, ns_exports)
-        stop(
-          "Linters ", glue::glue_collapse(sQuote(missing_linters), sep = ", ", last = " and "),
-          " advertised by `available_linters()` but not exported by package ", package, "."
-        )
+        missing_linters <- setdiff(available$linter, ns_exports) # nolint: object_usage_linter. TODO(#2252).
+        cli_abort(c(
+          x = "Can't find linters {.fn {missing_linters}}.",
+          i = "These are advertised by {.fn available_linters}, but are not exported by package {.pkg {package}}."
+        ))
       }
       linter_factories <- mget(available$linter, envir = pkg_ns)
       linters <- Map(
@@ -135,7 +139,7 @@ linters_with_tags <- function(tags, ..., packages = "lintr", exclude_tags = "dep
 #' - [available_linters] to get a data frame of available linters.
 #' - [linters] for a complete list of linters available in lintr.
 #' @export
-all_linters <- function(packages = "lintr", ...) {
+all_linters <- function(..., packages = "lintr") {
   linters_with_tags(tags = NULL, packages = packages, ...)
 }
 
@@ -145,15 +149,17 @@ all_linters <- function(packages = "lintr", ...) {
 #' The result of this function is meant to be passed to the `linters` argument of `lint()`,
 #' or to be put in your configuration file.
 #'
-#' @param defaults,default Default list of linters to modify. Must be named.
+#' @param defaults Default list of linters to modify. Must be named.
 #' @inheritParams linters_with_tags
-#' @examplesIf requireNamespace("withr", quietly = TRUE)
+#' @examples
 #' # When using interactively you will usually pass the result onto `lint` or `lint_package()`
-#' f <- withr::local_tempfile(lines = "my_slightly_long_variable_name <- 2.3", fileext = "R")
-#' lint(f, linters = linters_with_defaults(line_length_linter = line_length_linter(120)))
+#' f <- tempfile()
+#' writeLines("my_slightly_long_variable_name <- 2.3", f)
+#' lint(f, linters = linters_with_defaults(line_length_linter = line_length_linter(120L)))
+#' unlink(f)
 #'
 #' # the default linter list with a different line length cutoff
-#' my_linters <- linters_with_defaults(line_length_linter = line_length_linter(120))
+#' my_linters <- linters_with_defaults(line_length_linter = line_length_linter(120L))
 #'
 #' # omit the argument name if you are just using different arguments
 #' my_linters <- linters_with_defaults(defaults = my_linters, object_name_linter("camelCase"))
@@ -175,41 +181,19 @@ all_linters <- function(packages = "lintr", ...) {
 #' - [linters] for a complete list of linters available in lintr.
 #' @export
 linters_with_defaults <- function(..., defaults = default_linters) {
-  dots <- list(...)
-  if (missing(defaults) && "default" %in% names(dots)) {
-    warning(
-      "'default' is not an argument to linters_with_defaults(). Did you mean 'defaults'? ",
-      "This warning will be removed when with_defaults() is fully deprecated."
-    )
-    defaults <- dots$default
-    nms <- names2(dots)
-    missing_index <- !nzchar(nms, keepNA = TRUE)
-    if (any(missing_index)) {
-      names(dots)[missing_index] <- guess_names(..., missing_index = missing_index)
-    }
-    dots$default <- NULL
-    dots <- c(dots, list(defaults = defaults))
-    return(do.call(modify_defaults, dots))
-  }
   modify_defaults(..., defaults = defaults)
 }
 
-#' @rdname linters_with_defaults
-#' @export
-with_defaults <- function(..., default = default_linters) {
-  lintr_deprecated("with_defaults", "linters_with_defaults or modify_defaults", "3.0.0")
-  # to ease the burden of transition -- default = NULL used to behave like defaults = list() now does
-  if (is.null(default)) default <- list()
-  linters_with_defaults(..., defaults = default)
-}
-
 #' @keywords internal
 #' @noRd
 call_linter_factory <- function(linter_factory, linter_name, package) {
   linter <- tryCatch(
     linter_factory(),
     error = function(e) {
-      stop("Could not create linter with ", package, "::", linter_name, "(): ", conditionMessage(e))
+      cli_abort(
+        "Could not create linter with {.fun {package}::{linter_name}}.",
+        parent = e
+      )
     }
   )
   # Otherwise, all linters would be called "linter_factory".
@@ -220,14 +204,14 @@ call_linter_factory <- function(linter_factory, linter_name, package) {
 #' @keywords internal
 #' @noRd
 guess_names <- function(..., missing_index) {
-  args <- as.character(eval(substitute(alist(...)[missing_index])))
+  arguments <- as.character(eval(substitute(alist(...)[missing_index])))
   # foo_linter(x=1) => "foo"
   # var[["foo"]]    => "foo"
   # strip call: foo_linter(x=1) --> foo_linter
   # NB: Very long input might have newlines which are not caught
   #  by . in a perl regex; see #774
-  args <- re_substitutes(args, rex("(", anything), "", options = "s")
+  arguments <- re_substitutes(arguments, rex("(", anything), "", options = "s")
   # strip extractors: pkg::foo_linter, var[["foo_linter"]] --> foo_linter
-  args <- re_substitutes(args, rex(start, anything, '["' %or% "::"), "")
-  re_substitutes(args, rex('"]', anything, end), "")
+  arguments <- re_substitutes(arguments, rex(start, anything, '["' %or% "::"), "")
+  re_substitutes(arguments, rex('"]', anything, end), "")
 }
