@@ -47,29 +47,24 @@ yoda_test_linter <- function() {
   "
   pipes <- setdiff(magrittr_pipes, c("%$%", "%<>%"))
   xpath <- glue("
-  //SYMBOL_FUNCTION_CALL[text() = 'expect_equal' or text() = 'expect_identical' or text() = 'expect_setequal']
-    /parent::expr
-    /following-sibling::expr[1][ {const_condition} ]
+  following-sibling::expr[1][ {const_condition} ]
     /parent::expr[not(preceding-sibling::*[self::PIPE or self::SPECIAL[{ xp_text_in_table(pipes) }]])]
   ")
 
   second_const_xpath <- glue("expr[position() = 3 and ({const_condition})]")
 
-  Linter(function(source_expression) {
-    if (!is_lint_level(source_expression, "expression")) {
-      return(list())
-    }
-
-    xml <- source_expression$xml_parsed_content
-
-    bad_expr <- xml_find_all(xml, xpath)
+  Linter(linter_level = "expression", function(source_expression) {
+    bad_expr <- xml_find_all(
+      source_expression$xml_find_function_calls(c("expect_equal", "expect_identical", "expect_setequal")),
+      xpath
+    )
 
     matched_call <- xp_call_name(bad_expr)
     second_const <- xml_find_first(bad_expr, second_const_xpath)
     lint_message <- ifelse(
       is.na(second_const),
       paste(
-        "Tests should compare objects in the order 'actual', 'expected', not the reverse.",
+        "Compare objects in tests in the order 'actual', 'expected', not the reverse.",
         sprintf("For example, do %1$s(foo(x), 2L) instead of %1$s(2L, foo(x)).", matched_call)
       ),
       sprintf("Avoid storing placeholder tests like %s(1, 1)", matched_call)
