@@ -26,6 +26,7 @@ default_linters <- modify_defaults(
   object_name_linter(),
   object_usage_linter(),
   paren_body_linter(),
+  pipe_consistency_linter(),
   pipe_continuation_linter(),
   quotes_linter(),
   return_linter(),
@@ -166,6 +167,7 @@ default_undesirable_functions <- all_undesirable_functions[names(all_undesirable
   NULL
 )]
 
+# nocov start
 rd_auto_link <- function(x) {
   x <- unlist(x)
   x <- gsub(R"{([a-zA-Z0-9.]+)::([a-zA-Z0-9._]+)\(\)}", R"(\\code{\\link[\1:\2]{\1::\2()}})", x)
@@ -187,6 +189,7 @@ rd_undesirable_functions <- function() {
     "}"
   )
 }
+# nocov end
 
 #' @rdname default_undesirable_functions
 #' @format NULL
@@ -222,11 +225,12 @@ default_undesirable_operators <- all_undesirable_operators[names(all_undesirable
   NULL
 )]
 
+# nocov start
 rd_undesirable_operators <- function() {
   op_link_map <- c(
     `:::` = "\\link[base:ns-dblcolon]{:::}",
     `<<-` = "\\link[base:assignOps]{<<-}",
-    `->>` = "\\link[base:assignOps]{<<-}"
+    `->>` = "\\link[base:assignOps]{->>}"
   )
   op <- names(default_undesirable_operators)
 
@@ -242,6 +246,7 @@ rd_undesirable_operators <- function() {
     "}"
   )
 }
+# nocov end
 
 #' Default lintr settings
 #'
@@ -255,7 +260,6 @@ rd_undesirable_operators <- function() {
 #'  - `exclude_linter`, `exclude_linter_sep`: patterns used to exclude linters
 #'  - `exclusions`: a list of exclusions, see [exclude()] for a complete description of valid values.
 #'  - `cache_directory`: location of cache directory
-#'  - `comment_token`: a GitHub token character
 #'  - `error_on_lint`: decides if error should be produced when any lints are found
 #'
 #' There are no settings without defaults, i.e., this list describes every valid setting.
@@ -287,6 +291,14 @@ default_settings <- NULL
 settings <- new.env(parent = emptyenv())
 
 # nocov start
+logical_env <- function(x, unset = "") {
+  res <- as.logical(Sys.getenv(x, unset = unset))
+  if (is.na(res)) {
+    return(NULL)
+  }
+  res
+}
+
 .onLoad <- function(libname, pkgname) {
   op <- options()
   op_lintr <- list(
@@ -316,16 +328,7 @@ settings <- new.env(parent = emptyenv())
     exclude_linter_sep = rex(any_spaces, ",", any_spaces),
     exclusions = list(),
     cache_directory = R_user_dir("lintr", "cache"),
-    comment_token = Sys.getenv("GITHUB_TOKEN", unset = NA) %||% rot(
-      paste0(
-        "0n12nn72507",
-        "r6273qnnp34",
-        "43qno7q42n1",
-        "n71nn28"
-      ),
-      54L - 13L
-    ),
-    error_on_lint = logical_env("LINTR_ERROR_ON_LINT") %||% FALSE
+    error_on_lint = logical_env("LINTR_ERROR_ON_LINT", unset = FALSE)
   ))
 
   reset_settings()
