@@ -7,10 +7,14 @@
 # paren_brace
 f = function (x,y = 1){}
 
+# return_linter
+g <- function(x) {
+  return(x + 1)
+}
+
 # commented_code
 # some <- commented("out code")
 
-# cyclocomp
 # equals_na
 # brace_linter
 # indentation
@@ -35,6 +39,7 @@ my_metric <- function(x)
   sum(x) + prod(x)
 
 # no_tab
+# pipe_consistency
 # pipe_continuation
 # seq_linter
 # spaces_inside
