@@ -2,18 +2,18 @@ test_that("expect_comparison_linter skips allowed usages", {
   linter <- expect_comparison_linter()
 
   # there's no expect_ne() for this operator
-  expect_lint("expect_true(x != y)", NULL, linter)
+  expect_no_lint("expect_true(x != y)", linter)
   # NB: also applies to tinytest, but it's sufficient to test testthat
-  expect_lint("testthat::expect_true(x != y)", NULL, linter)
+  expect_no_lint("testthat::expect_true(x != y)", linter)
 
   # multiple comparisons are OK
-  expect_lint("expect_true(x > y || x > z)", NULL, linter)
+  expect_no_lint("expect_true(x > y || x > z)", linter)
 
   # expect_gt() and friends don't have an info= argument
-  expect_lint("expect_true(x > y, info = 'x is bigger than y yo')", NULL, linter)
+  expect_no_lint("expect_true(x > y, info = 'x is bigger than y yo')", linter)
 
   # expect_true() used incorrectly, and as executed the first argument is not a lint
-  expect_lint("expect_true(is.count(n_draws), n_draws > 1)", NULL, linter)
+  expect_no_lint("expect_true(is.count(n_draws), n_draws > 1)", linter)
 })
 
 test_that("expect_comparison_linter blocks simple disallowed usages", {
@@ -49,6 +49,15 @@ test_that("expect_comparison_linter blocks simple disallowed usages", {
     rex::rex("expect_identical(x, y) is better than expect_true(x == y)."),
     linter
   )
+
+  expect_lint(
+    trim_some("
+      expect_true(x # comment
+      == (y == 2))
+    "),
+    rex::rex("expect_identical(x, y) is better than expect_true(x == y)."),
+    expect_comparison_linter()
+  )
 })
 
 test_that("lints vectorize", {
