@@ -2,20 +2,19 @@ test_that("returns the correct linting", {
   linter <- object_usage_linter()
   local_var_msg <- rex::rex("local variable", anything, "assigned but may not be used")
 
-  expect_lint("blah", NULL, linter)
+  expect_no_lint("blah", linter)
 
-  expect_lint(
+  expect_no_lint(
     trim_some("
       function() {
         a <- 1
         a
       }
     "),
-    NULL,
     linter
   )
 
-  expect_lint(
+  expect_no_lint(
     trim_some("
       fun <- function(x) {
         fun(1)
@@ -24,7 +23,6 @@ test_that("returns the correct linting", {
         fun2(2)
       }
     "),
-    NULL,
     linter
   )
 
@@ -154,22 +152,20 @@ test_that("replace_functions_stripped", {
 })
 
 test_that("eval errors are ignored", {
-  expect_lint(
+  expect_no_lint(
     trim_some('
     setMethod("[[<-", c("stampedEnv", "character", "missing"),
       function(x) {
         x
       })
     '),
-    NULL,
     object_usage_linter()
   )
 })
 
 test_that("calls with top level function definitions are ignored", {
-  expect_lint(
+  expect_no_lint(
     'tryCatch("foo", error = function(e) e)',
-    NULL,
     object_usage_linter()
   )
 })
@@ -191,7 +187,7 @@ test_that("object-usage line-numbers are relative to start-of-file", {
 
 test_that("used symbols are detected correctly", {
   # From #666
-  expect_lint(
+  expect_no_lint(
     trim_some('
       foo <- data.frame(0)
       foo$bar <- 1
@@ -200,7 +196,6 @@ test_that("used symbols are detected correctly", {
       }
       message(zero())
     '),
-    NULL,
     object_usage_linter()
   )
 
@@ -217,7 +212,7 @@ test_that("used symbols are detected correctly", {
   )
 
   # Also test deeper nesting
-  expect_lint(
+  expect_no_lint(
     trim_some('
       foo <- list(0)
       foo$bar$baz$goo <- 1
@@ -229,12 +224,11 @@ test_that("used symbols are detected correctly", {
       }
       message(zero())
     '),
-    NULL,
     object_usage_linter()
   )
 
   # Test alternative assignment and access methods
-  expect_lint(
+  expect_no_lint(
     trim_some('
       foo <- list(0)
       foo[["bar"]][["baz"]][["goo"]] <- 1
@@ -249,14 +243,12 @@ test_that("used symbols are detected correctly", {
       }
       message(zero())
     '),
-    NULL,
     object_usage_linter()
   )
 
 
-
   # regression #1322
-  expect_silent(expect_lint("assign('x', 42)", NULL, object_usage_linter()))
+  expect_silent(expect_no_lint("assign('x', 42)", object_usage_linter()))
 })
 
 test_that("object_usage_linter finds lints spanning multiple lines", {
@@ -312,7 +304,7 @@ test_that("global variable detection works", {
   utils::globalVariables("global_function", package = globalenv())
   on.exit(utils::globalVariables(old_globals, package = globalenv(), add = FALSE))
 
-  expect_lint(
+  expect_no_lint(
     trim_some("
       foo <- function() {
         if (global_function()) NULL
@@ -322,7 +314,6 @@ test_that("global variable detection works", {
         }
       }
     "),
-    NULL,
     object_usage_linter()
   )
 })
@@ -330,14 +321,13 @@ test_that("global variable detection works", {
 test_that("package detection works", {
   expect_length(
     lint_package(test_path("dummy_packages", "package"), linters = object_usage_linter(), parse_settings = FALSE),
-    9L
+    10L
   )
 })
 
 test_that("robust against errors", {
-  expect_lint(
+  expect_no_lint(
     'assign("x", unknown_function)',
-    NULL,
     object_usage_linter()
   )
 })
@@ -345,38 +335,62 @@ test_that("robust against errors", {
 test_that("interprets glue expressions", {
   linter <- object_usage_linter()
 
-  expect_lint(trim_some("
-    fun <- function() {
-      local_var <- 42
-      glue::glue('The answer is {local_var}.')
-    }
-  "), NULL, linter)
+  expect_no_lint(
+    trim_some("
+      fun <- function() {
+        local_var <- 42
+        glue::glue('The answer is {local_var}.')
+      }
+    "),
+    linter
+  )
+
+  # no need for namespace-qualification
+  expect_no_lint(
+    trim_some("
+      glue <- glue::glue # imitate this being an @import
+      fun <- function() {
+        local_var <- 42
+        glue('The answer is {local_var}.')
+      }
+    "),
+    linter
+  )
 
   # multiple variables in different interpolations
-  expect_lint(trim_some("
-    fun <- function() {
-      local_key <- 'a'
-      local_value <- 123
-      glue::glue('Key-value pair: {local_key}={local_value}.')
-    }
-  "), NULL, linter)
+  expect_no_lint(
+    trim_some("
+      fun <- function() {
+        local_key <- 'a'
+        local_value <- 123
+        glue::glue('Key-value pair: {local_key}={local_value}.')
+      }
+    "),
+    linter
+  )
 
   # multiple variables in single interpolation
-  expect_lint(trim_some("
-    fun <- function() {
-      local_str1 <- 'a'
-      local_str2 <- 'b'
-      glue::glue('With our powers combined: {paste(local_str1, local_str2)}.')
-    }
-  "), NULL, linter)
+  expect_no_lint(
+    trim_some("
+      fun <- function() {
+        local_str1 <- 'a'
+        local_str2 <- 'b'
+        glue::glue('With our powers combined: {paste(local_str1, local_str2)}.')
+      }
+    "),
+    linter
+  )
 
   # Check non-standard .open and .close
-  expect_lint(trim_some("
-    fun <- function() {
-      local_var <- 42
-      glue::glue('The answer is $[local_var].', .open = '$[', .close = ']')
-    }
-  "), NULL, linter)
+  expect_no_lint(
+    trim_some("
+      fun <- function() {
+        local_var <- 42
+        glue::glue('The answer is $[local_var].', .open = '$[', .close = ']')
+      }
+    "),
+    linter
+  )
 
   # Steer clear of custom .transformer and .envir constructs
   expect_lint(trim_some("
@@ -409,7 +423,32 @@ test_that("interprets glue expressions", {
       local_var <- 42
       glue::glue('The answer is {local_var}.')
     }
-  "), "local_var", object_usage_linter(interpret_glue = FALSE))
+  "), "local_var", object_usage_linter(interpret_extensions = NULL))
+
+  # call in glue is caught
+  expect_lint(
+    trim_some("
+      fun <- function() {
+        local_call <- identity
+        local_unused_call <- identity
+        glue::glue('{local_call(1)}')
+      }
+    "),
+    "local_unused_call",
+    linter
+  )
+
+  # ditto infix operator
+  expect_no_lint(
+    trim_some("
+      glue <- glue::glue # imitate this being an @import
+      foo <- function() {
+        `%++%` <- `+`
+        glue('{x %++% y}')
+      }
+    "),
+    linter
+  )
 })
 
 test_that("errors/edge cases in glue syntax don't fail lint()", {
@@ -417,7 +456,7 @@ test_that("errors/edge cases in glue syntax don't fail lint()", {
 
   # no lint & no error, despite glue error
   expect_warning(
-    expect_lint(
+    expect_no_lint(
       trim_some("
         fun <- function() {
           a <- 2
@@ -425,7 +464,6 @@ test_that("errors/edge cases in glue syntax don't fail lint()", {
           glue::glue('The answer is {a')
         }
       "),
-      NULL,
       linter
     ),
     "Evaluating glue expression.*failed: Expecting '\\}'.*Please ensure correct glue syntax"
@@ -459,19 +497,18 @@ test_that("errors/edge cases in glue syntax don't fail lint()", {
   )
 
   # empty glue expression {}
-  expect_lint(
+  expect_no_lint(
     trim_some("
       fun <- function() {
         a <- 2
         glue::glue('The answer is {}: {a}')
       }
     "),
-    NULL,
     linter
   )
 
   # comment inside glue range (#1919)
-  expect_lint(
+  expect_no_lint(
     trim_some("
       fun <- function() {
         a <- 2
@@ -480,7 +517,6 @@ test_that("errors/edge cases in glue syntax don't fail lint()", {
         )
       }
     "),
-    NULL,
     linter
   )
 })
@@ -508,7 +544,7 @@ test_that("backtick'd names in glue are handled", {
 
 # reported as #1088
 test_that("definitions below top level are ignored (for now)", {
-  expect_lint(
+  expect_no_lint(
     trim_some("
       local({
         x <- 1
@@ -517,7 +553,6 @@ test_that("definitions below top level are ignored (for now)", {
         }
       })
     "),
-    NULL,
     object_usage_linter()
   )
 })
@@ -536,7 +571,7 @@ test_that("package imports are detected if present in file", {
     object_usage_linter()
   )
 
-  expect_lint(
+  expect_no_lint(
     trim_some("
       library(xml2)
 
@@ -545,7 +580,6 @@ test_that("package imports are detected if present in file", {
         a
       }
     "),
-    NULL,
     object_usage_linter()
   )
 })
@@ -617,7 +651,10 @@ test_that("missing libraries don't cause issue", {
         a
       }
     "),
-    NULL,
+    list(
+      "Could not find exported symbols for package \"a.a.a.z.z.z\"",
+      line_number = 1L
+    ),
     object_usage_linter()
   )
 })
@@ -641,6 +678,8 @@ test_that("messages without a quoted name are caught", {
 
 # See #1914
 test_that("symbols in formulas aren't treated as 'undefined global'", {
+  linter <- object_usage_linter()
+
   expect_lint(
     trim_some("
       foo <- function(x) {
@@ -655,7 +694,7 @@ test_that("symbols in formulas aren't treated as 'undefined global'", {
       line_number = 4L,
       column_number = 21L
     ),
-    object_usage_linter()
+    linter
   )
 
   # neither on the RHS
@@ -673,7 +712,7 @@ test_that("symbols in formulas aren't treated as 'undefined global'", {
       line_number = 4L,
       column_number = 21L
     ),
-    object_usage_linter()
+    linter
   )
 
   # nor in nested expressions
@@ -691,7 +730,7 @@ test_that("symbols in formulas aren't treated as 'undefined global'", {
       line_number = 4L,
       column_number = 21L
     ),
-    object_usage_linter()
+    linter
   )
 
   # nor as a call
@@ -712,11 +751,29 @@ test_that("symbols in formulas aren't treated as 'undefined global'", {
       line_number = 4L,
       column_number = 21L
     ),
-    object_usage_linter()
+    linter
+  )
+
+  # native lambda requires being in an expression to support a comment immediately after
+  expect_lint(
+    trim_some("
+      foo <- \\ # comment
+      (x) {
+        lm(
+          y(w) ~ z,
+          data = x[!is.na(y)]
+        )
+      }
+    "),
+    "no visible",
+    linter
   )
 })
 
 test_that("NSE-ish symbols after $/@ are ignored as sources for lints", {
+  linter <- object_usage_linter()
+  lint_msg <- "no visible binding for global variable 'column'"
+
   expect_lint(
     trim_some("
       foo <- function(x) {
@@ -726,12 +783,8 @@ test_that("NSE-ish symbols after $/@ are ignored as sources for lints", {
         )
       }
     "),
-    list(
-      message = "no visible binding for global variable 'column'",
-      line_number = 4L,
-      column_number = 22L
-    ),
-    object_usage_linter()
+    list(lint_msg, line_number = 4L, column_number = 22L),
+    linter
   )
 
   expect_lint(
@@ -743,12 +796,8 @@ test_that("NSE-ish symbols after $/@ are ignored as sources for lints", {
         )
       }
     "),
-    list(
-      message = "no visible binding for global variable 'column'",
-      line_number = 4L,
-      column_number = 22L
-    ),
-    object_usage_linter()
+    list(lint_msg, line_number = 4L, column_number = 22L),
+    linter
   )
 })
 
@@ -767,53 +816,34 @@ test_that("functional lambda definitions are also caught", {
 })
 
 test_that("messages without location info are repaired", {
+  linter <- object_usage_linter()
+  global_function_msg <- rex::rex("no visible global function definition for", anything)
+  global_variable_msg <- rex::rex("no visible binding for global variable", anything)
+  local_variable_msg <- rex::rex("local variable", anything, "assigned but may not be used")
+
   # regression test for #1986
   expect_lint(
-    trim_some("
-      foo <- function() no_fun()
-    "),
-    list(
-      message = rex::rex("no visible global function definition for", anything),
-      line_number = 1L,
-      column_number = 19L
-    ),
-    object_usage_linter()
+    "foo <- function() no_fun()",
+    list(global_function_msg, line_number = 1L, column_number = 19L),
+    linter
   )
 
   expect_lint(
-    trim_some("
-      foo <- function(a = no_fun()) a
-    "),
-    list(
-      message = rex::rex("no visible global function definition for", anything),
-      line_number = 1L,
-      column_number = 21L
-    ),
-    object_usage_linter()
+    "foo <- function(a = no_fun()) a",
+    list(global_function_msg, line_number = 1L, column_number = 21L),
+    linter
   )
 
   expect_lint(
-    trim_some("
-      foo <- function() no_global
-    "),
-    list(
-      message = rex::rex("no visible binding for global variable", anything),
-      line_number = 1L,
-      column_number = 19L
-    ),
-    object_usage_linter()
+    "foo <- function() no_global",
+    list(global_variable_msg, line_number = 1L, column_number = 19L),
+    linter
   )
 
   expect_lint(
-    trim_some("
-      foo <- function() unused_local <- 42L
-    "),
-    list(
-      message = rex::rex("local variable", anything, "assigned but may not be used"),
-      line_number = 1L,
-      column_number = 19L
-    ),
-    object_usage_linter()
+    "foo <- function() unused_local <- 42L",
+    list(local_variable_msg, line_number = 1L, column_number = 19L),
+    linter
   )
 
   # More complex case with two lints and missing location info
@@ -823,17 +853,78 @@ test_that("messages without location info are repaired", {
         bar()
     "),
     list(
-      list(
-        message = rex::rex("local variable", anything, "assigned but may not be used"),
-        line_number = 1L,
-        column_number = 19L
-      ),
-      list(
-        message = rex::rex("no visible global function definition for", anything),
-        line_number = 2L,
-        column_number = 3L
-      )
+      list(local_variable_msg, line_number = 1L, column_number = 19L),
+      list(global_function_msg, line_number = 2L, column_number = 3L)
     ),
-    object_usage_linter()
+    linter
   )
 })
+
+test_that("globals in scripts are found regardless of assignment operator", {
+  linter <- object_usage_linter()
+
+  expect_no_lint(
+    trim_some("
+      library(dplyr)
+
+      global_const_eq = 5
+      global_const_la <- 6
+      7 -> global_const_ra
+
+      examplefunction <- function(df) {
+        df %>%
+          select(dist) %>%
+          mutate(power = global_const_eq + global_const_ra + global_const_la)
+      }
+    "),
+    linter
+  )
+})
+
+test_that("dplyr's .env-specified objects are marked as 'used'", {
+  skip_if_not_installed("dplyr")
+  skip_if_not_installed("rlang")
+  linter <- object_usage_linter()
+
+  expect_lint( # nofuzz
+    trim_some("
+      foo <- function(df) {
+        source <- 1
+        target <- 2
+        unused <- 3
+
+        df %>%
+          dplyr::mutate(
+            from = rlang::.env$source,
+            to = rlang::.env[['target']]
+          )
+      }
+    "),
+    rex::rex("local variable", anything, "unused"),
+    linter
+  )
+})
+
+test_that("interpret_glue is deprecated", {
+  expect_warning(
+    {
+      linter_no <- object_usage_linter(interpret_glue = FALSE)
+    },
+    rex::rex("interpret_glue", anything, "deprecated")
+  )
+  expect_warning(
+    {
+      linter_yes <- object_usage_linter(interpret_glue = TRUE)
+    },
+    rex::rex("interpret_glue", anything, "deprecated")
+  )
+
+  code <- trim_some("
+    fun <- function() {
+      local_var <- 42
+      glue::glue('The answer is {local_var}.')
+    }
+  ")
+  expect_lint(code, "local_var", linter_no)
+  expect_no_lint(code, linter_yes)
+})
