@@ -1,16 +1,23 @@
 test_that("redundant_equals_linter skips allowed usages", {
+  linter <- redundant_equals_linter()
+
   # comparisons to non-logical constants
-  expect_lint("x == 1", NULL, redundant_equals_linter())
+  expect_no_lint("x == 1", linter)
   # comparison to TRUE as a string
-  expect_lint("x != 'TRUE'", NULL, redundant_equals_linter())
+  expect_no_lint("x != 'TRUE'", linter)
 })
 
-test_that("mutliple lints return correct custom messages", {
+test_that("multiple lints return correct custom messages", {
   expect_lint(
-    "list(x == TRUE, y != TRUE)",
+    trim_some("
+      list(
+        x == TRUE,
+        y != TRUE
+      )
+    "),
     list(
-      "Using == on a logical vector",
-      "Using != on a logical vector"
+      list("Using == on a logical vector", line_number = 2L),
+      list("Using != on a logical vector", line_number = 3L)
     ),
     redundant_equals_linter()
   )
@@ -35,3 +42,14 @@ patrick::with_parameters_test_that(
     "!=, FALSE", "!=", "FALSE"
   )
 )
+
+test_that("logic survives adversarial comments", {
+  expect_lint(
+    trim_some("
+      list(x #
+      == TRUE)
+    "),
+    "==",
+    redundant_equals_linter()
+  )
+})
